/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.maven;

import ch.hortis.sonar.model.MavenProject;
import ch.hortis.sonar.model.ProjectLink;
import ch.hortis.sonar.model.Snapshot;
import ch.hortis.sonar.mvn.CoreMojo;
import ch.hortis.sonar.mvn.Report;
import org.apache.maven.model.CiManagement;
import org.apache.maven.model.IssueManagement;
import org.apache.maven.model.Scm;
import org.apache.maven.plugin.MojoExecutionException;
import org.picocontainer.MutablePicoContainer;
import org.sonar.commons.DaoFacade;
import org.sonar.commons.database.DatabaseManager;
import org.sonar.plugins.api.Languages;
import org.sonar.plugins.api.MavenCollector;
import org.sonar.plugins.api.Plugin;
import org.sonar.plugins.java.JavaMeasuresRecorder;
import org.sonar.plugins.maven.MavenUtils;
import org.sonar.plugins.plsql.PlsqlMeasuresRecorder;

import javax.persistence.NoResultException;
import javax.persistence.Query;
import java.util.Date;
import java.util.List;

/**
 * @goal collect
 */
public class CollectMojo extends CoreMojo {

  public void doExecute() throws MojoExecutionException {
    Snapshot snapshot = createSnapshot(mavenProject, new Date(), getContainer().getComponent(DaoFacade.class).getDatabaseManager());

    MutablePicoContainer childContainer = null;
    try {
      childContainer = getContainer().makeChildContainer();
      childContainer.addComponent(snapshot);
      childContainer.addComponent(JavaMeasuresRecorder.class);
      childContainer.addComponent(PlsqlMeasuresRecorder.class);
      for (Plugin plugin : getContainer().getComponents(Plugin.class)) {
        if (plugin.getMavenCollectorClass() != null) {
          childContainer.addComponent(plugin.getMavenCollectorClass());
        }
      }
      if (MavenUtils.getLanguage(mavenProject).equals(Languages.JAVA)) {
        registerDeprecatedCollectors(childContainer);
      }
      childContainer.start();

      for (MavenCollector analyser : childContainer.getComponents(MavenCollector.class)) {
        try {
          if (analyser.shouldCollectOn(mavenProject)) {
            analyser.collect(mavenProject);
          }
        } catch (RuntimeException e) {
          if (analyser.shouldStopOnFailure()) {
            throw e;
          } else {
            getLog().error("error while analysing " + analyser.getClass().getSimpleName());
          }
        }
      }

      for (Collector collector : childContainer.getComponents(Collector.class)) {
        try {
          getLog().debug("collecting " + collector.getClass().getSimpleName());
          if (collector.shouldExecuteOn(mavenProject)) {
            collector.collect();
          }

        } catch (RuntimeException e) {
          if (collector.shouldStopOnFailure()) {
            throw e;
          } else {
            getLog().error("error while collecting " + collector.getClass().getSimpleName());
          }
        }
      }

    } catch (Exception e) {
      throw new MojoExecutionException("Error during sonar plugin access", e);

    } finally {
      if (childContainer != null) {
        //childContainer.stop();
        getContainer().removeChildContainer(childContainer);
      }
    }

    setSnapshotId(snapshot.getId(), mavenProject);
  }

  private void registerDeprecatedCollectors(MutablePicoContainer container) {
    for (Report report : getTargetReports()) {
      if (report.getCollectorClass() != null) {
        container.addComponent(report.getCollectorClass());
      }
    }
  }

  private List<Report> getTargetReports() {
    if (isLight()) {
      return Report.getLightReports();
    }
    return Report.getReports();
  }

  private Snapshot createSnapshot(org.apache.maven.project.MavenProject pom, Date date, DatabaseManager dbManager) {
    MavenProject dbProject = loadProjectFromDatabase(pom.getGroupId(), pom.getArtifactId(), MavenUtils.getBranch(pom), dbManager);
    if (dbProject == null) {
      dbProject = MavenProject.newMavenProject(pom.getGroupId(), pom.getArtifactId(), MavenUtils.getBranch(pom), pom.getName());
      dbProject.setQualifier(MavenUtils.getLanguage(pom).getProjectQualifier());

    } else {
      //dbProject.getProjectLinks().size();
    }
    dbProject.setName(pom.getName());
    dbProject.setDescription(pom.getDescription());
    dbProject = (MavenProject) dbManager.save(dbProject);

    updateProject(dbProject, pom);

    Snapshot snapshot = new Snapshot();
    snapshot.setMavenProject(dbProject);
    snapshot.setVersion(pom.getVersion());
    snapshot.setCreatedAt(date);
    dbManager.save(snapshot);
    return snapshot;
  }

  private MavenProject loadProjectFromDatabase(String groupId, String artifactId, String branch, DatabaseManager dbManager) {
    try {
      Query query = dbManager.getEntityManager().createQuery("SELECT p FROM MavenProject p WHERE p.scope=:scope AND p.key=:key AND p.enabled=true");
      query.setParameter("scope", MavenProject.SCOPE_PROJECT);
      query.setParameter("key", MavenProject.toMavenKey(groupId, artifactId, branch));
      return (MavenProject) query.getSingleResult();

    } catch (NoResultException e) {
      return null;
    }
  }

  private void updateProject(MavenProject sonarProject, org.apache.maven.project.MavenProject pom) {
    updateProjectLink(ProjectLink.LINK_HOME_PAGE, pom.getUrl(), sonarProject);
    Scm scmConfig = pom.getScm();
    if (scmConfig == null) {
      scmConfig = new Scm();
    }
    updateProjectLink(ProjectLink.LINK_SCM_URL, scmConfig.getUrl(), sonarProject);
    updateProjectLink(ProjectLink.LINK_SCM_DEV_CONNECTION, scmConfig.getDeveloperConnection(), sonarProject);
    updateProjectLink(ProjectLink.LINK_SCM_RO_CONNECTION, scmConfig.getConnection(), sonarProject);

    CiManagement cimConfig = pom.getCiManagement();
    if (cimConfig == null) {
      cimConfig = new CiManagement();
    }
    updateProjectLink(ProjectLink.LINK_CONTINUOUS_INTEGRATION, cimConfig.getUrl(), sonarProject);

    IssueManagement imConfig = pom.getIssueManagement();
    if (imConfig == null) {
      imConfig = new IssueManagement();
    }
    updateProjectLink(ProjectLink.LINK_ISSUES_TRACKER, imConfig.getUrl(), sonarProject);
  }

  private void updateProjectLink(String linkType, String href, MavenProject sonarProject) {
    if (href != null && !"".equals(href)) {
      ProjectLink link = sonarProject.getProjectLinkByType(linkType);
      if (link == null) {
        link = new ProjectLink();
        link.setMavenProject(sonarProject);
        link.setType(linkType);
        sonarProject.getProjectLinks().add(link);
      }
      link.setHref(href);
    }
  }
}
